
@==========================================================================@
/*    A GIBBS SAMPLING APPROACH TO A REGRESSION WITH AUTOCORRELATED ERRORS:
         Y_t = BETA_0 + BETA_1*X_t + e_t,
           e_t = phi e_{t-1} + v_t
                     v_t -- i.i.d. N(0, sigma^2)

           based on Albert and Chibs'(Journal of Econometrics, 1993)
       Subprograms.

*/
@==========================================================================@


     new;
     library pgraph;

     format /m1 /rd 9,4;

     @ GENERATING DATA@
     /* DATA GENERATING PROCESS:

                  y_t = b0 + b1*x_t + e_t
             x_t = ro*x_{t-1} + u_t, u_t --iid N(0,v_x2)
             e_t = rho*e_{t-1} + v_t, v_t -- iid N(0,v_e2) */

T=150;   b0=0; b1=1; ro=0.5; v_x2=1; rho=0.5; v_e2=1;


     seed1=33333;
     rndns(1,1,seed1);

     x_mat=zeros(t,1);
     y_mat=zeros(t,1);
     e_mat=zeros(t,1);

     v_mat=rndn(t,1)*sqrt(v_e2);
     u_mat=rndn(t,1)*sqrt(v_x2);

     e_mat[1,1]= rndn(1,1)*sqrt(v_e2/(1-rho^2));
     x_mat[1,1]=rndn(1,1)*sqrt(v_x2/(1-ro^2));

     y_mat[1,1]= b0 + b1*x_mat[1,1]+ e_mat[1,1];

     i=2; do until i>T;

        e_mat[i,1]=rho*e_mat[i-1,1]+v_mat[i,1];
        x_mat[i,1]=ro*x_mat[i-1,1]+u_mat[i,1];
        y_mat[i,1]=b0 + b1*x_mat[i,1] + e_mat[i,1];

     i=i+1;
     endo;

Y=Y_MAT;
X=ONES(T,1)~X_MAT;

output file=gbs_ato.dta reset;
y_mat~x_mat; output off;

         @================= PARAMETERS FOR PRIOR DISTRIBUTIONS====@

     R0=EYE(2)*1000000;
     BETA0=0|0;        @FOR beta0,beta1@

     R00=EYE(1)*1000000;
     phi0=0;        @FOR  phi @

     nu0=0;
     d0=0;                    @FOR SIG2_V OF INDIVIDUAL COMPONENT@


         @========INITIAL VALUES FOR THE GIBBS SAMPLER ===========@


       SIG2=0.5;
       phi=0;


         @===== DESIGN  FOR THE GIBBS SAMPLER ===========@

       N0 = 1000;       @NUMBER OF DRAWS TO LEAVE OUT@
       MM = 4000;       @NUMBER OF DRAWS@

       CAPN = N0 + MM;  @TOTAL NUMBER OF DRAWS@


         @===== STORAGE SPACES===========================@


           SIG2MM = {};
           BETAMM = {};
           phimm  ={};


   @ BEGIN ITERATION========================================@

  ITR=1;
  DO WHILE ITR LE CAPN;      ITR;;"th  iteration.......";

     BETA=GEN_BETA;
     SIG2=GEN_SIG2;
     phi =gen_phi;


    @==========END OF ONE ITERATION ========================@


    IF ITR>N0;

       SIG2MM = SIG2MM~SIG2;
       BETAMM = BETAMM~BETA;
       phimm=phimm~phi;

    ENDIF;

  ITR=ITR+1;
  ENDO;

  FNL_MAT=BETAMM'~phimm'~SIG2MM';
  output file=data.dta reset; fnl_mat; output off;

     @======== calculation of posterior expectations and std deviations@


          INDX = ZEROS(MM,1);
          INDX[1] = 1;
          J = 1;
          DO WHILE J LE MM;
               INDX[J] = 1;
          J = J + 5;
          ENDO;


          SORT_OUT=ZEROS(cols(fnl_mat),3);
          I_NDX=1;
          DO UNTIL I_NDX>COLS(FNL_MAT);

                tmpm1 = selif(FNL_MAT[.,I_NDX],indx);
                MN_OUT = meanc(tmpm1);
                STD_OUT = stdc(tmpm1);
                MED_OUT = median(tmpm1);
                u2out = MN_OUT~STD_OUT~MED_OUT;

        SORT_OUT[I_NDX,.]=U2OUT;

           I_NDX=I_NDX+1;
       ENDO;

OUTPUT FILE=GBS_ato.OUT RESET;

       "FROM 1st TO --nd ROWS..............";

       "For beta0, beta1,phi,sigma^2";
       "===================================";
       "mean, sd, median";
       SORT_OUT;

       OUTPUT OFF;



xy(seqa(1,1,rows(betamm[1,.]')),betamm[1,.]');
{a,b,c}=histp(betamm[1,.]',50);

xy(seqa(1,1,rows(betamm[2,.]')),betamm[2,.]');
{a,b,c}=histp(betamm[2,.]',50);

xy(seqa(1,1,rows(phimm')),phimm');
{a,b,c}=histp(phimm',50);

xy(seqa(1,1,rows(sig2mm')),sig2mm');
{a,b,c}=histp(sig2mm',50);

END;



     @=============================================================@
     @ GENERATING BETA (beta0,beta)@
     @=============================================================@

PROC GEN_BETA;

     LOCAL V, BETA1, BETA_F, C, ACCEPT, COEF, ROOT, ROOTMOD,xstar,ystar,
           xstar0;

     ystar= y[2:t,1]- phi*y[1:t-1,.];
     xstar= x[2:t,.]- phi*x[1:t-1,.];

     V = inv(INV(R0) + SIG2^(-1)*XSTAR'XSTAR);
     BETA1 =  V*(INV(R0)*BETA0 + SIG2^(-1)*XSTAR'YSTAR);
     C = chol(V);

          BETA_F = BETA1 + C'rndn(2,1); @GENERATE BETA@

RETP(BETA_F);
ENDP;

     @=============================================================@
     @ GENERATING PHI@
     @=============================================================@

PROC GEN_PHI;

     LOCAL V, PHI1, PHI_F, C, ACCEPT, COEF, ROOT, ROOTMOD,xstar,ystar,
           estar;

     estar= y- x*beta;
     ystar=estar[2:t,1];
     xstar=estar[1:t-1,1];

     V = inv(INV(R00) + SIG2^(-1)*XSTAR'XSTAR);
     PHI1 =  V*(INV(R00)*PHI0 + SIG2^(-1)*XSTAR'YSTAR);
     C = chol(V);

     ACCEPT = 0;
     DO WHILE ACCEPT ==0;

          PHI_F = PHI1 + C'rndn(1,1); @GENERATE BETA@
          COEF = -REV(PHI_F)|1;
          ROOT = POLYROOT(COEF);
          ROOTMOD = ABS(ROOT);

          IF MINC(ROOTMOD) GE 1.0001;
              ACCEPT = 1;
          ELSE;
              ACCEPT = 0;
          ENDIF;

    ENDO;


RETP(PHI_F);
ENDP;


     @===============================================================@
     @ GENERATING SIG2 CONDITIONAL ON BETA,PHI                       @
     @===============================================================@

PROC GEN_SIG2;
     LOCAL NU1, D1,C,T2,SIG2_F,estar,ystar,xstar,tstar,ss;

     estar= y- x*beta;
     ystar=estar[2:t,1];
     xstar=estar[1:t-1,1];
     ss= (ystar-xstar*phi)'(ystar-xstar*phi);

     tstar=rows(ystar);

     NU1 = t + NU0;
     D1 = D0 + ss;

     c = rndc(NU1);
     t2 = c/D1;
     SIG2_F = 1/t2;

RETP(SIG2_f);
ENDP;



@========================================================================@
@========================================================================@
 /* ADDPROCS.GSI */
 /* compiled file used in Albert and Chib (JBES, 1993, 11,1-15) */
 /* please contact Siddhartha Chib at chib@olin.wustl.edu
            if there are problems*/

proc rndc(a);       /* chisquare */
local x,w;

a = a/2;
w= rg1(a);
x = w * 2;

retp(x);
endp;


proc rndb(a,b);                /* beta */
local x,a1n,a1d;

a1n = rg1(a);
a1d = rg1(b);
x = a1n / (a1n + a1d);

retp(x);
endp;


proc rg1(a);
local x,j,w,u;

if a gt 1;
x = rg2(a);
elseif a lt 1;
a = a + 1;
u = rndu(1,1);
x = rg2(a)*u^(1/a);
elseif a == 1;
x = -ln(rndu(1,1));
endif;

retp(x);
endp;


proc rg2(a);
local gam,accept,b,c,j,x,z,u,v,w,y;

b = a-1;
c = 3*a - .75;
accept = 0;
do while accept == 0;
u = rndu(1,1);
v = rndu(1,1);
w = u*(1-u);
y = sqrt(c/w)*(u-.5);
x = b+y;
if x ge 0;
z = 64*(w^3)*(v^2);
accept = z le ( 1-(2*y^2)/x );
if accept == 0;
accept = ln(z) le 2*(b*ln(x/b) - y);
endif;
endif;
endo;

retp(x);
endp;

